#!/bin/bash

echo "\nOTUX-parser (v-1.0b) - a parser for output generated by mothur when using OTUX as a database.\n" >& 2
echo "Biosciences R&D, Tata Consultancy Services Limited\n\n" >& 2


if [ "$#" -lt 4 ]; then
	echo "\nUsage:\n\t./otux_wrapper.sh [Arguments]\n" >& 2
	echo "\n\t-m\t-\ttaxonomy output file generated by mothur [mandatory]\n" >& 2
	echo "\n\t-b\t-\tmapping back file for the corresponding v-region [mandatory]\n" >& 2
	echo "\n\t-g\t-\tgreengenes taxonomy file [mandatory]\n" >& 2
	echo "\n\t-f\t-\tfasta file used as input for mothur [mandatory]\n" >& 2
	exit 1
fi

while getopts ":m:b:g:f:" opt; do
	case $opt in
		m) mothur_out="$OPTARG" ;;
		b) mapback="$OPTARG" ;;
		g) gg_tax="$OPTARG" ;;
		f) fasta="$OPTARG" ;;
		\?) echo "Invalid option -$OPTARG" >&2 ;;
	esac
done

if [ -z $mothur_out ]; then
    echo "\nError!! Mandatory Argument missing! Enter mothur output file name.\n"
    exit 1
fi

if [ -z $mapback ]; then
    echo "\nError!! Mandatory Argument missing! Enter mapping back file name.\n"
    exit 1
fi

if [ -z $gg_tax ]; then
    echo "\nError!! Mandatory Argument missing! Enter greengenes taxonomy file.\n"
    exit 1
fi

if [ -z $fasta ]; then
    echo -e "\nError!! Mandatory Argument missing! Enter fasta file name used as input for mothur.\n"
    exit 1
fi

if [ ! -f $mothur_out ]; then
	echo "\nError:\tFile mothur_out not found\n" >& 2
	exit 1
fi

if [ ! -f $mapback ]; then
	echo "\nError:\tFile $mapback not found\n" >& 2
	exit 1
fi

if [ ! -f $gg_tax ]; then
	echo "\nError:\tFile $gg_tax not found\n" >& 2
	exit 1
fi

if [ ! -f $fasta ]; then
	echo "\nError:\tFile $fasta not found\n" >& 2
	exit 1
fi

if [ ! -f bin/parseMothur ]; then
	echo "\nError:\tparseMothur executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

if [ ! -f bin/mapbackOTU ]; then
	echo "\nError:\tmapbackOTU executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

if [ ! -f bin/mapbackTaxa ]; then
	echo "\nError:\tmapbackTaxa executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

if [ ! -f bin/unclassifiedSeqs ]; then
	echo "\nError:\tunclassifiedSeqs executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

if [ ! -f bin/mothur2json ]; then
	echo "\nError:\tmothur2json executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

if [ ! -f bin/percentNormalize ]; then
	echo "\nError:\tpercentNormalize executable not found in the 'bin/' directory\n" >& 2
	exit 1
fi

name=`echo $mothur_out | cut -f1 -d'.'`

./bin/parseMothur $mothur_out
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing parseMothur binary\n" >& 2
	exit 1
fi

mainDir=$name"_output_"`date | awk '{print $4}'`
mkdir $mainDir
dir1=$mainDir"/OTUX_abundances"
mkdir $dir1

dir1_1=$dir1"/RawCount"
mkdir $dir1_1
dir1_2=$dir1"/PercentNorm"
mkdir $dir1_2

./bin/percentNormalize phylum.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv phylum.txt $dir1_1"/"$name"_otux_raw_phylum.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_phylum.txt"

./bin/percentNormalize class.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv class.txt $dir1_1"/"$name"_otux_raw_class.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_class.txt"

./bin/percentNormalize order.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv order.txt $dir1_1"/"$name"_otux_raw_order.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_order.txt"

./bin/percentNormalize family.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv family.txt $dir1_1"/"$name"_otux_raw_family.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_family.txt"

./bin/percentNormalize genus.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv genus.txt $dir1_1"/"$name"_otux_raw_genus.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_genus.txt"

./bin/percentNormalize species.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv species.txt $dir1_1"/"$name"_otux_raw_species.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_species.txt"

./bin/percentNormalize fullLineage.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv fullLineage.txt $dir1_1"/"$name"_otux_raw_fullLineage.txt"
mv normalized.txt $dir1_2"/"$name"_otux_norm_fullLineage.txt"


./bin/percentNormalize otu.txt
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "\nError in executing percentNormalize binary\n" >& 2
	exit 1
fi
mv normalized.txt $dir1_2"/"$name"_otux_norm_otu.txt"

./bin/mapbackOTU otu.txt $mapback
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing mapbackOTU binary\n" >& 2
	exit 1
fi

mv otu.txt $dir1_1"/"$name"_otux_raw_otu.txt"

./bin/mapbackTaxa otu_mapBackONE.txt $gg_tax
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing mapbackTaxa binary while performing one-to-one mapping back \n" >& 2
	exit 1
fi

dir2=$mainDir"/Greengenes_abundances"
mkdir $dir2

dir2_1=$dir2"/OneToOne_mapback"
mkdir $dir2_1
dir2_2=$dir2"/OneToMany_mapback"
mkdir $dir2_2

mv phylum.txt $dir2_1"/"$name"_gg_OneToOne_phylum.txt"
mv class.txt $dir2_1"/"$name"_gg_OneToOne_class.txt"
mv order.txt $dir2_1"/"$name"_gg_OneToOne_order.txt"
mv family.txt $dir2_1"/"$name"_gg_OneToOne_family.txt"
mv genus.txt $dir2_1"/"$name"_gg_OneToOne_genus.txt"
mv species.txt $dir2_1"/"$name"_gg_OneToOne_species.txt"
mv fullLineage.txt $dir2_1"/"$name"_gg_OneToOne_fullLineage.txt"
mv otu_mapBackONE.txt $dir2_1"/"$name"_gg_OneToOne_otu.txt"

./bin/mapbackTaxa otu_mapBackMANY.txt $gg_tax
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing mapbackTaxa binary while performing one-to-many mapping back \n" >& 2
	exit 1
fi

mv phylum.txt $dir2_2"/"$name"_gg_OneToMany_phylum.txt"
mv class.txt $dir2_2"/"$name"_gg_OneToMany_class.txt"
mv order.txt $dir2_2"/"$name"_gg_OneToMany_order.txt"
mv family.txt $dir2_2"/"$name"_gg_OneToMany_family.txt"
mv genus.txt $dir2_2"/"$name"_gg_OneToMany_genus.txt"
mv species.txt $dir2_2"/"$name"_gg_OneToMany_species.txt"
mv fullLineage.txt $dir2_2"/"$name"_gg_OneToMany_fullLineage.txt"
mv otu_mapBackMANY.txt $dir2_2"/"$name"_gg_OneToMany_otu.txt"

./bin/mothur2json $mothur_out
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing mothur2json binary \n" >& 2
	exit 1
fi

mv taxonomy.json $mainDir"/"$name"_taxonomy.json"

./bin/unclassifiedSeqs $mothur_out $fasta
exitCode=$?

if [ $exitCode -ne 0 ]; then
	echo "\nError in executing unclassifiedSeqs binary \n" >& 2
	exit 1
fi


num=`grep -c '>' unclassified.fasta`

echo "\nOutput files created in $mainDir\n"  >& 2

if [ $num -eq 0 ]; then
	rm unclassified.fasta
else
	mv unclassified.fasta $mainDir"/"$name"_unclassified.fasta"
	echo "\n$num sequences were not classified till OTU level. A fasta file containing these sequences has been created in $mainDir. You may use CROP or CD-HIT for de-novo clustering of these sequences.\n"  >& 2
fi


